from experiment.luna.pipeline.luna_gold_transcript_processor import LunaGoldTranscriptProcessor
from experiment.luna.pipeline.pos_processing.asr_spacy_token_pos_processing_task import \
    AsrSpacyTokenPosProcessingTask
from experiment.luna.pipeline.pos_processing.gold_transcript_spacy_token_pos_processing_task import \
    GoldTranscriptSpacyTokenPosProcessingTask
from experiment.luna.pipeline.pos_processing.spacy_pos_wer_processing_task import \
    SpacyPosWerProcessingTask
from experiment.luna.luna_record_provider import LunaRecordProvider
from sziszapangma.integration.asr_processor import AsrPathCacheClient
from sziszapangma.integration.experiment_manager import ExperimentManager
from sziszapangma.integration.path_filter import ExtensionPathFilter
from sziszapangma.integration.repository.experiment_repository import ExperimentRepository
from sziszapangma.integration.repository.multi_files_experiment_repository import \
    MultiFilesExperimentRepository
from sziszapangma.integration.task.asr_task import AsrTask
from sziszapangma.integration.task.classic_wer_metric_task import ClassicWerMetricTask
from sziszapangma.integration.task.gold_transcript_task import GoldTranscriptTask

LUNA_DIRECTORY = 'experiment_data/dataset/LUNA.PL'
GOLD_TRANSCRIPT = 'gold_transcript'
TECHMO_POLISH_ASR = 'techmo_polish_asr'
TECHMO_POLISH_CLASSIC_WER_METRIC = 'techmo_polish_classic_wer_metric'
TECHMO_POLISH_CLASSIC_ALIGNMENT = 'techmo_polish_classic_alignment'
TECHMO_SPACY = 'techmo_spacy'
GOLD_TRANSCRIPT_SPACY = 'gold_transcript_spacy'
POS_ALIGNMENT_WER = 'pos_alignment_wer'
POS_METRICS_WER = 'pos_metrics_wer'


def run_luna_experiment(experiment_repository: ExperimentRepository):
    record_provider = LunaRecordProvider(
        ExtensionPathFilter(
            root_directory=LUNA_DIRECTORY,
            extension='wav'
        ),
        relation_manager_root_path='experiment_data/dataset_relation_manager_data/luna'
    )
    experiment_processor = ExperimentManager(
        record_id_iterator=record_provider,
        processing_tasks=[
            GoldTranscriptTask(
                task_name='gold_transcript_task',
                gold_transcript_processor=LunaGoldTranscriptProcessor(record_provider),
                gold_transcript_property_name=GOLD_TRANSCRIPT,
                require_update=False
            ),
            AsrTask(
                task_name='techmo_polish_task',
                # asr_processor=AsrWebClient('http://192.168.0.124:4999/process_asr', 'test1234'),
                asr_processor=AsrPathCacheClient('experiment_data/cached_asr/luna_techmo', record_provider,
                                                 record_provider),
                asr_property_name=TECHMO_POLISH_ASR,
                require_update=False,
                record_path_provider=record_provider
            ),
            ClassicWerMetricTask(
                task_name='classic_wer_metric_task',
                asr_property_name=TECHMO_POLISH_ASR,
                gold_transcript_property_name=GOLD_TRANSCRIPT,
                metrics_property_name=TECHMO_POLISH_CLASSIC_WER_METRIC,
                require_update=False,
                alignment_property_name=TECHMO_POLISH_CLASSIC_ALIGNMENT
            ),
            GoldTranscriptSpacyTokenPosProcessingTask(
                task_name='gold_transcript_spacy_task',
                input_property_name=GOLD_TRANSCRIPT,
                spacy_property_name=GOLD_TRANSCRIPT_SPACY,
                require_update=True
            ),
            AsrSpacyTokenPosProcessingTask(
                task_name='techmo_spacy_task',
                input_property_name=TECHMO_POLISH_ASR,
                spacy_property_name=TECHMO_SPACY,
                require_update=True
            ),
            SpacyPosWerProcessingTask(
                task_name='PosWerProcessor',
                require_update=False,
                gold_transcript_pos_property_name=GOLD_TRANSCRIPT_SPACY,
                asr_pos_property_name=TECHMO_SPACY,
                pos_alignment_wer=POS_ALIGNMENT_WER,
                pos_metrics_wer=POS_METRICS_WER
            )
        ],
        experiment_repository=experiment_repository,
        relation_manager_provider=record_provider
    )
    experiment_processor.process()


def example_run():
    experiment_repository = MultiFilesExperimentRepository(
        'experiment_data/pipeline', 'asr_benchmark_luna')
    run_luna_experiment(experiment_repository)


if __name__ == '__main__':
    example_run()
